import { LuciaAdapter } from "./abstract.js";
import { DrizzleSQLiteAdapter } from "@lucia-auth/adapter-drizzle";
import sqlite from "better-sqlite3";
import { sqliteTable, text, integer } from "drizzle-orm/sqlite-core";
import { drizzle, } from "drizzle-orm/better-sqlite3";
import { eq, sql } from "drizzle-orm";
const userTable = sqliteTable("user", {
    id: text("id").primaryKey(),
    subject: text("subject").notNull().unique(),
    claims: text("claims").notNull(),
});
const sessionTable = sqliteTable("session", {
    id: text("id").primaryKey(),
    userId: text("user_id")
        .notNull()
        .references(() => userTable.id),
    expiresAt: integer("expires_at").notNull(),
    idToken: text("id_token").notNull(),
});
class SqliteDrizzleAdapter extends LuciaAdapter {
    db;
    constructor() {
        const sqliteDB = sqlite(":memory:");
        const db = drizzle(sqliteDB);
        const adapter = new DrizzleSQLiteAdapter(db, sessionTable, userTable);
        super(adapter, {
            getUserAttributes(attributes) {
                return {
                    subject: attributes.subject,
                    claims: JSON.parse(attributes.claims),
                };
            },
        });
        this.db = db;
    }
    initDatabase() {
        this.db.run(sql `CREATE TABLE IF NOT EXISTS user (
      id TEXT NOT NULL PRIMARY KEY,
      subject TEXT NOT NULL UNIQUE,
      claims TEXT NOT NULL
    )`);
        this.db.run(sql `CREATE TABLE IF NOT EXISTS session (
      id TEXT NOT NULL PRIMARY KEY,
      expires_at INTEGER NOT NULL,
      user_id TEXT NOT NULL,
      id_token TEXT NOT NULL,
      FOREIGN KEY (user_id) REFERENCES user(id)
    )`);
    }
    async getUserId(subject) {
        const user = await this.db
            .select()
            .from(userTable)
            .where((user) => eq(user.subject, subject));
        return user.at(0)?.id;
    }
    async newUser(subject, userId, claims) {
        await this.db
            .insert(userTable)
            .values([{ id: userId, subject, claims: JSON.stringify(claims) }]);
        return userId;
    }
    newSession(userId, tokenSet) {
        return this.createSession(userId, {
            idToken: tokenSet.id_token,
        });
    }
    getIdToken(session) {
        return session.idToken;
    }
}
export const adapter = new SqliteDrizzleAdapter();
