import { redirect, error } from "@sveltejs/kit";
import { sequence } from "@sveltejs/kit/hooks";
import { generateId } from "lucia";
import { Issuer, errors, generators } from "openid-client";
import { SEE_OTHER } from "readable-http-codes";
import { method, route, setLuciaCookie, setTempCookie, addBasePath, } from "./helpers.js";
export class AriseIdConnect {
    config;
    client;
    paths;
    cookieNames;
    constructor(config, issuer) {
        this.config = config;
        this.client = new issuer.Client({
            ...config,
            grant_types: ["authorization_code", "refresh_token"],
            response_types: ["code", "id_token"],
        });
        this.cookieNames = {
            oauthState: config.cookieNames?.oauthState ?? "aidc_state",
            oauthCodeVerifier: config.cookieNames?.oauthCodeVerifier ?? "aidc_code_verifier",
        };
        this.paths = {
            home: addBasePath(config.paths?.home ?? "/"),
            callback: addBasePath(config.paths?.callback ?? "/auth/callback"),
            logoutCallback: addBasePath(config.paths?.logoutCallback ?? "/"),
            login: addBasePath(config.paths?.login ?? "/auth/login"),
            logout: addBasePath(config.paths?.logout ?? "/auth/logout"),
        };
    }
    static async init(config) {
        const issuer = await Issuer.discover(config.issuer ||
            "https://oidc.iiens.net/.well-known/openid-configuration");
        const aidc = new AriseIdConnect(config, issuer);
        return aidc;
    }
    async getTokens(event) {
        const state = event.cookies.get(this.cookieNames.oauthState) ?? null;
        const codeVerifier = event.cookies.get(this.cookieNames.oauthCodeVerifier) ?? null;
        const params = this.client.callbackParams(event.url.toString());
        if (!codeVerifier || !state) {
            redirect(SEE_OTHER, this.paths.login);
        }
        const redirectURI = new URL(this.paths.callback, event.url).toString();
        try {
            const tokenSet = await this.client.callback(redirectURI, params, {
                code_verifier: codeVerifier,
                state: state,
            });
            return tokenSet;
        }
        catch (err) {
            if (err instanceof errors.OPError) {
                if (err.error === "access_denied") {
                    if (this.config.on?.accessDenied) {
                        return this.config.on.accessDenied(event, err);
                    }
                    redirect(SEE_OTHER, this.paths.home);
                }
                error(500, err);
            }
            else if (err instanceof errors.RPError) {
                error(500, err);
            }
            throw err;
        }
    }
    loginHandler = ({ event }) => {
        const codeVerifier = generators.codeVerifier();
        const state = generators.state();
        // const nonce = generators.nonce();
        const redirectURI = new URL(this.paths.callback, event.url).toString();
        const authorizationUrl = this.client.authorizationUrl({
            response_type: "code",
            scope: this.config.scope,
            code_challenge: generators.codeChallenge(codeVerifier),
            code_challenge_method: "S256",
            redirect_uri: redirectURI,
            state,
            // nonce,
        });
        setTempCookie(event, this.cookieNames.oauthState, state);
        setTempCookie(event, this.cookieNames.oauthCodeVerifier, codeVerifier);
        redirect(SEE_OTHER, authorizationUrl);
    };
    logoutHandler = async ({ event }) => {
        if (!event.locals.session) {
            redirect(SEE_OTHER, this.paths.home);
        }
        const { adapter: lucia } = this.config;
        const { session } = await lucia.validateSession(event.locals.session.id);
        const postLogoutRedirectURI = new URL(this.paths.logoutCallback, event.url);
        const idToken = await lucia.getIdToken(session);
        const endSessionUrl = this.client.endSessionUrl({
            post_logout_redirect_uri: postLogoutRedirectURI.toString(),
            id_token_hint: idToken,
        });
        if (this.config.on?.logout) {
            await this.config.on.logout(event);
        }
        await lucia.invalidateSession(event.locals.session.id);
        setLuciaCookie(event, lucia.createBlankSessionCookie());
        redirect(SEE_OTHER, endSessionUrl);
    };
    callbackHandler = async ({ event }) => {
        const tokenSet = await this.getTokens(event);
        if (!tokenSet.id_token) {
            throw new Error("No id_token in tokenSet");
        }
        const claims = tokenSet.claims();
        const { sub } = claims;
        const lucia = this.config.adapter;
        const existingUserId = await lucia.getUserId(sub);
        let userId = existingUserId ?? generateId(15);
        if (existingUserId === undefined) {
            userId = await lucia.newUser(sub, userId, claims);
        }
        const session = await lucia.newSession(userId, tokenSet);
        setLuciaCookie(event, lucia.createSessionCookie(session.id));
        if (this.config.on?.login) {
            return this.config.on.login(event, tokenSet.claims());
        }
        redirect(SEE_OTHER, this.paths.home);
    };
    sessionHandler = async ({ event, resolve }) => {
        const lucia = this.config.adapter;
        const sessionId = event.cookies.get(lucia.sessionCookieName);
        if (!sessionId) {
            event.locals.user = null;
            event.locals.session = null;
            return resolve(event);
        }
        const { session, user } = await lucia.validateSession(sessionId);
        if (session && session.fresh) {
            setLuciaCookie(event, lucia.createSessionCookie(session.id));
        }
        if (!session) {
            setLuciaCookie(event, lucia.createBlankSessionCookie());
        }
        event.locals.user = user;
        event.locals.session = session;
        return resolve(event);
    };
    miscHandler = ({ event, resolve }) => {
        event.locals.authPaths = this.paths;
        return resolve(event);
    };
    handler() {
        return sequence(this.miscHandler.bind(this), route(this.paths.login, this.loginHandler.bind(this)), route(this.paths.logout, method("POST", this.logoutHandler.bind(this))), route(this.paths.callback, this.callbackHandler.bind(this)), this.sessionHandler.bind(this));
    }
}
